<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://example.com
 * @since      1.0.0
 *
 * @package    WP_Fitsales
 * @subpackage WP_Fitsales/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WP_Fitsales
 * @subpackage WP_Fitsales/admin
 * @author     David De Franchi <david@pragmatica.com.ar>
 */
class WP_Fitsales_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Plugin_Name_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Plugin_Name_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/wp-fitsales-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Plugin_Name_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Plugin_Name_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/wp-fitsales-admin.js', array( 'jquery' ), $this->version, false );

	}

    public function add_admin_menu() {
        //add_menu_page( $page_title, $menu_title, $capability, $menu_slug, $function, $icon_url, $position );
        add_menu_page(  'WP Fitsales < Formularios', 'Fitsales', 'administrator', $this->plugin_name, array( $this, 'display_admin_dashboard' ), 'dashicons-id-alt', 26 );
        
        //add_submenu_page( '$parent_slug, $page_title, $menu_title, $capability, $menu_slug, $function );
        add_submenu_page( $this->plugin_name, 'WP Fitsales < Formularios', 'Formularios', 'administrator', $this->plugin_name, array( $this, 'display_admin_dashboard' ));
        add_submenu_page( $this->plugin_name, 'WP Fitsales < Configuración', 'Configuración', 'administrator', $this->plugin_name.'-settings', array( $this, 'display_admin_settings' ));
    }

    public function display_admin_dashboard() {
        // set this var to be used in the settings-display view
        $active_tab = isset( $_GET[ 'tab' ] ) ? $_GET[ 'tab' ] : 'general';
        if(isset($_GET['error_message'])){
            add_action('admin_notices', array($this,'admin_messages'));
            do_action( 'admin_notices', $_GET['error_message'] );
        }

        require_once 'partials/demoversion.php';
        require_once 'partials/'.$this->plugin_name.'-admin-display.php';
    }

    public function display_admin_settings() {
        // set this var to be used in the settings-display view
        $active_tab = isset( $_GET[ 'tab' ] ) ? $_GET[ 'tab' ] : 'general';
        if(isset($_GET['error_message'])){
            add_action('admin_notices', array($this,'settings_messages'));
            do_action( 'admin_notices', $_GET['error_message'] );
        }

        require_once 'partials/demoversion.php';
        require_once 'partials/'.$this->plugin_name.'-admin-settings-display.php';
    }

    public function settings_messages($error_message){
        switch ($error_message) {
            case '1':
                $message = __( 'There was an error adding this setting. Please try again.  If this persists, shoot us an email.', 'my-text-domain' );                 
                $err_code = esc_attr( 'wp_fitsales_example_setting' );                 
                $setting_field = 'wp_fitsales_example_setting';                 
                break;
        }
        $type = 'error';
        add_settings_error(
               $setting_field,
               $err_code,
               $message,
               $type
           );
    }

    public function register_fields() {
        /**
         * First, we add_settings_section. This is necessary since all future settings must belong to one.
         * Second, add_settings_field
         * Third, register_setting
         */     
        add_settings_section(
            // ID used to identify this section and with which to register options
            'wp_fitsales_general_section', 
            // Title to be displayed on the administration page
            '',  
            // Callback used to render the description of the section
            array( $this, 'wp_fitsales_display_general_account' ),    
            // Page on which to add this section of options
            'wp_fitsales_general_settings'                   
        );

        /*----ENDPOINT---*/
        unset($args);

        $args = array (
            'type'      => 'input',
            'subtype'   => 'text',
            'id'    => 'wp_fitsales_endpoint',
            'name'      => 'wp_fitsales_endpoint',
            'required'  => true,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'prepend_value' => "https://",
            'append_value' => ".fitsales.com.ar",
        );

        add_settings_field(
            'wp_fitsales_endpoint',
            'URL Envío',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_general_settings',
            'wp_fitsales_general_section',
            $args
        );

        register_setting(
            'wp_fitsales_general_settings',
            'wp_fitsales_endpoint',
            [
                'type' => 'string',
                'sanitize_callback' => array( $this, 'sanitize_endpoint_option')
            ]
        );

        /*----TOKEN---*/
        unset($args);

        $args = array (
            'type'      => 'input',
            'subtype'   => 'text',
            'id'    => 'wp_fitsales_token',
            'name'      => 'wp_fitsales_token',
            'required'  => true,
            'value_type'=>'normal',
            'wp_data' => 'option'
        );

        add_settings_field(
            'wp_fitsales_token',
            'Token Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_general_settings',
            'wp_fitsales_general_section',
            $args
        );

        register_setting(
            'wp_fitsales_general_settings',
            'wp_fitsales_token'
        );

        /*-----------------*/
        /*--FORMS SECTION--*/
        /*-----------------*/

        if (WP_Fitsales_Public::is_cf7_integrated()) {
            $this->generate_cf7_section();
        }

        if (WP_Fitsales_Public::is_wpforms_integrated()) {
            $this->generate_wpforms_section();
        }

        if (WP_Fitsales_Public::is_ninjaforms_integrated()) {
            $this->generate_ninjaforms_section();
        }

        //var_dump(WP_Fitsales_Public::get_matched_fields(WP_Fitsales_Public::getPrefixCF7(), 127));
    }

    public function sanitize_endpoint_option($data) {
        
        if ( !WP_FITSALES_SANDBOX)
            $data = stripslashes(str_replace('/', '', str_replace('.', '', $data)));

        return $data;
    }

    public function wp_fitsales_display_general_account() {
        echo '<p>Agregue los datos de Fitsales para vincular sus formularios al CRM</p>';
    }

    public function wp_fitsales_render_settings_field($args) {
        /* EXAMPLE INPUT
                  'type'      => 'input',
                  'subtype'   => '',
                  'id'    => $this->plugin_name.'_example_setting',
                  'name'      => $this->plugin_name.'_example_setting',
                  'required'  => true,
                  'get_options_list' => "",
                    'value_type' = serialized OR normal,
        'wp_data'=>(option or post_meta),
        'post_id' =>
        */

        if($args['wp_data'] == 'option'){
            $wp_data_value = get_option($args['name']);
        } elseif($args['wp_data'] == 'post_meta'){
            $wp_data_value = get_post_meta($args['post_id'], $args['name'], true );
        }
        $required = $args['required'] ? 'required="required"' : '';

        switch ($args['type']) {

            case 'input':
                $value = ($args['value_type'] == 'serialized') ? serialize($wp_data_value) : $wp_data_value;
                if($args['subtype'] != 'checkbox'){
                    $prepostvalue = isset($args['prepend_value']) || isset($args['append_value']);
                    $prependStart = ($prepostvalue) ? '<div class="input-prepend"> <span class="add-on">' . (isset($args['prepend_value']) ? $args['prepend_value'] : '') . ' </span>' : '';
                    $prependEnd = ($prepostvalue) ? (isset($args['append_value']) ? '<span class="add-on"> ' . $args['append_value'] . '</span>' : '') . ' </div>' : '';
                    $step = (isset($args['step'])) ? 'step="'.$args['step'].'"' : '';
                    $min = (isset($args['min'])) ? 'min="'.$args['min'].'"' : '';
                    $max = (isset($args['max'])) ? 'max="'.$args['max'].'"' : '';
                    if(isset($args['disabled'])){
                        // hide the actual input bc if it was just a disabled input the informaiton saved in the database would be wrong - bc it would pass empty values and wipe the actual information
                        echo $prependStart.'<input type="'.$args['subtype'].'" id="'.$args['id'].'_disabled" '.$step.' '.$max.' '.$min.' name="'.$args['name'].'_disabled" size="40" disabled value="' . esc_attr($value) . '" /><input type="hidden" id="'.$args['id'].'" '.$step.' '.$max.' '.$min.' name="'.$args['name'].'" size="40" value="' . esc_attr($value) . '" />'.$prependEnd;
                    } else {
                        echo $prependStart.'<input type="'.$args['subtype'].'" id="'.$args['id'].'" '.$required . $step.' '.$max.' '.$min.' name="'.$args['name'].'" size="40" value="' . esc_attr($value) . '" />'.$prependEnd;
                    }

                } else {
                    $checked = ($value) ? 'checked' : '';
                    echo '<input type="'.$args['subtype'].'" id="'.$args['id'].'" ' . $required . ' name="'.$args['name'].'" size="40" value="1" '.$checked.' />';
                }

                if (isset($args['tip']))
                    echo '<p class="tip">' . $args['tip'] . '</p>';

                break;
            case 'textarea':
                if (isset($args['pretip']))
                    echo '<p class="pretip pretip-textarea">' . $args['pretip'] . '</p>';
                $value = $wp_data_value;
                $prependStart = (isset($args['prepend_value'])) ? '<div class="input-prepend"> <span class="add-on">'.$args['prepend_value'].'</span>' : '';
                $prependEnd = (isset($args['prepend_value'])) ? '</div>' : '';
                echo $prependStart . '<textarea cols="38" rows="4" id="' . $args['id'] . '" ' . $required . ' name="'.$args['name'].'">' . esc_attr($value) . '</textarea>' . $prependEnd;

                if (isset($args['tip']))
                    echo '<p class=" tip-textarea">' . $args['tip'] . '</p>';

                break;
            case 'select':
                $existingvalue = $wp_data_value;
                $fieldlist = $args['get_options_list'];

                if (!is_array($fieldlist) && $fieldlist=='cf7')
                {
                    $fieldlist = WP_Fitsales_Public::getFormFieldListCF7($args['post_id']);
                }

                echo '<select name="'.$args['id'].'" id="'.$args['name'].'">';
                echo '<option value="" ' . ($existingvalue=='' ? 'selected="selected"' : '') . ' >' . '(Ignorar)' . '</option>';
                foreach($fieldlist as $key => $value) {
                    echo '<option value="'.$key.'" ' . ($existingvalue == (string) $key ? 'selected="selected"' : '') . ' >'.$value.'</option>';
                }
                echo '</select>';
                
                break;
            default:
                # code...
                break;
        }
    }


    public function wp_fitsales_section_cfg_conector() {
        echo '<h2 class="titulo">Configuración de conector</h2>';
        echo '<p>Edite los datos que desee vincular de este formulario con Fitsales</p>';
    }

    public function wp_fitsales_section_fuente_origen() {
        echo '<h2 class="titulo">Identificación de fuente de origen</h2>';
        echo '<p>Vincula los datos de este formulario a Fitsales junto a los valores de origen como Fuente, Medio y Campaña.</p>';
    }

    public function wp_fitsales_section_utm() {
        echo '<h3 class="subtitulo">Vincular a UTMs preconfigurados</h3>';
        echo '<p>Active la vinculación de UTMs configurados en campañas de Google, Facebook, Instagram, etc. Además, en el formulario deben preconfigurarse como campos ocultos (utm_source, utm_medium y utm_campaign).</p>';
    }

    public function wp_fitsales_section_origen() {
        echo '<h3 class="subtitulo">Establecer el origen de datos</h3>';
        echo '<p>En caso de no activar UTMs, puede establecer los valores de origen en los siguientes campos. Es obligatorio completar la fuente con el mismo valor configurado en el conector de Fitsales.</p>';
    }

    public function wp_fitsales_section_matching_fields() {
        echo '<h2 class="titulo">Integración de Campos</h2>';
        echo '<p>Establezca la concordancia entre los campos de Fitsales y los de este formulario. Los campos que se ignoran no se enviarán.</p>';
    }

    public function wp_fitsales_section_datos_adicionales() {
        echo '<p>Agregue comentarios y/o campos adicionales del formulario. Defina los datos de los campos del formulario entre llaves, ej. {sitio_web} y puede identificarlos con un texto, o separarlos con comas, etc.</p>';
    }

    public function generate_cf7_section() {
        $forms = WP_Fitsales_Public::getBasicFormsCF7();
        $prefix = WP_Fitsales_Public::getPrefixCF7();

        foreach($forms as $form_id => $form_title) {
            //$fieldlist = WP_Fitsales_Public::getFormFieldListCF7($form_id);
            //como queda instanciado el form (WPCF7_ContactForm::get_instance) cuando llamo a getFormFieldListCF7 y da error CF7
            //devuelvo 'cf7' para que la función que renderiza el campo (en otra página) llame a la función. Ver línea 317.
            $fieldlist = 'cf7';
            $this->generate_common_sections($prefix, $form_id, $form_title, $fieldlist);
        }
    }

    public function generate_wpforms_section() {
        $forms = WP_Fitsales_Public::getBasicFormsWPForms();
        $prefix = WP_Fitsales_Public::getPrefixWPForms();

        foreach($forms as $form_id => $form_title) {
            $fieldlist = WP_Fitsales_Public::getFormFieldListWPForms($form_id);
            $this->generate_common_sections($prefix, $form_id, $form_title, $fieldlist);
        }
    }

    public function generate_ninjaforms_section() {
        $forms = WP_Fitsales_Public::getBasicFormsNinjaForms();
        $prefix = WP_Fitsales_Public::getPrefixNinjaForms();
        
        foreach($forms as $form_id => $form_title) {
            $fieldlist = WP_Fitsales_Public::getFormFieldListNinjaForms($form_id);
            $this->generate_common_sections($prefix, $form_id, $form_title, $fieldlist);
        }
    }

    public function generate_common_sections($prefix, $id, $title, $fieldlist) {

        //array de relaciones, cada campo que agrego lo debo relacionar a uno de fitsales.
        //este array se crea al activarse el plugin. Ver clase activator.
        $relations = get_option('wp_fitsales_fields_relations');
        
        //Sección de Configuracion: campaña, fuente y medio
        add_settings_section(
            // ID used to identify this section and with which to register options
            'wp_fitsales_section_cfg_' . $prefix . '_' . $id,
            // Title to be displayed on the administration page
            '',  
            // Callback used to render the description of the section
            array( $this, 'wp_fitsales_section_cfg_conector' ),
            // Page on which to add this section of options
            'wp_fitsales_settings_' . $prefix . '_' . $id
        );

        /*----ACTIVO---*/
        unset($args);
        $fieldname = 'wp_fitsales_cfg_activo_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'input',
            'subtype'   => 'checkbox',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required' => false,
            'value_type'=>'normal',
            'wp_data' => 'option',
        );

        add_settings_field(
            $fieldname,
            'Vincular a Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_cfg_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----CONECTOR---*/
        unset($args);
        $fieldname = 'wp_fitsales_cfg_conector_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'input',
            'subtype'   => 'number',
            'min'       => 1,
            'id'        => $fieldname,
            'name'      => $fieldname,
            'required'  => true,
            'value_type'=>'normal',
            'wp_data'   => 'option'
        );

        add_settings_field(
            $fieldname,
            'ID Conector Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_cfg_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        //Sección de Origen:
        add_settings_section(
            // ID used to identify this section and with which to register options
            'wp_fitsales_section_fuente_origen' . $prefix . '_' . $id,
            // Title to be displayed on the administration page
            '',
            // Callback used to render the description of the section
            array( $this, 'wp_fitsales_section_fuente_origen' ),
            // Page on which to add this section of options
            'wp_fitsales_settings_' . $prefix . '_' . $id
        );

        //Sección de UTM:
        add_settings_section(
            // ID used to identify this section and with which to register options
            'wp_fitsales_section_utm_' . $prefix . '_' . $id,
            // Title to be displayed on the administration page
            '',
            // Callback used to render the description of the section
            array( $this, 'wp_fitsales_section_utm' ),
            // Page on which to add this section of options
            'wp_fitsales_settings_' . $prefix . '_' . $id
        );

        /*----UTM---*/
        unset($args);
        $fieldname = 'wp_fitsales_cfg_utm_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'input',
            'subtype'   => 'checkbox',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required' => false,
            'value_type'=>'normal',
            'wp_data' => 'option',
            //'tip' => 'Se intentarán tomar los valores de Fuente, Medio y Campaña en campos ocultos:  utm_source, utm_medium y utm_campaign.<br />De lo contrario, se utilizará por defecto la información de los campos a continuación.',
        );

        add_settings_field(
            $fieldname,
            'Activar UTMs',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_utm_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        //Sección de Origen de Datos:
        add_settings_section(
            // ID used to identify this section and with which to register options
            'wp_fitsales_section_origen_' . $prefix . '_' . $id,
            // Title to be displayed on the administration page
            '',
            // Callback used to render the description of the section
            array( $this, 'wp_fitsales_section_origen' ),
            // Page on which to add this section of options
            'wp_fitsales_settings_' . $prefix . '_' . $id
        );

        /*----FUENTE---*/
        unset($args);
        $fieldname = 'wp_fitsales_cfg_fuente_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'input',
            'subtype'   => 'text',
            'min'       => 1,
            'id'        => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'value_type'=>'normal',
            'wp_data'   => 'option'
        );

        if (!in_array($fieldname, $relations['fuente_fitsales'])) {
            array_push($relations['fuente_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Fuente',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_origen_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----MEDIO---*/
        unset($args);
        $fieldname = 'wp_fitsales_cfg_medio_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'input',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'value_type'=>'normal',
            'wp_data' => 'option'
        );

        if (!in_array($fieldname, $relations['medio_fitsales'])) {
            array_push($relations['medio_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Medio',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_origen_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----CAMPAÑA---*/
        unset($args);
        $fieldname = 'wp_fitsales_cfg_campania_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'input',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'value_type'=>'normal',
            'wp_data' => 'option'
        );

        if (!in_array($fieldname, $relations['campania_fitsales'])) {
            array_push($relations['campania_fitsales'], $fieldname);
        }
        
        add_settings_field(
            $fieldname,
            'Campaña',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_origen_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        //Seccion de Matching
        add_settings_section(
            // ID used to identify this section and with which to register options
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            // Title to be displayed on the administration page
            '',  
            // Callback used to render the description of the section
            array( $this, 'wp_fitsales_section_matching_fields' ),
            // Page on which to add this section of options
            'wp_fitsales_settings_' . $prefix . '_' . $id
        );

        /*----NOMBRE---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_nombre_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => true,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['nombre_fitsales'])) {
            array_push($relations['nombre_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Nombre Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----APELLIDO---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_apellido_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['apellido_fitsales'])) {
            array_push($relations['apellido_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Apellido Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----COMPAÑÍA---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_compania_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['compania_fitsales'])) {
            array_push($relations['compania_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Compañía Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----CARGO---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_cargo_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['cargo_fitsales'])) {
            array_push($relations['cargo_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Cargo Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----EMAIL---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_email_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['email_fitsales'])) {
            array_push($relations['email_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Email Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----TEL LABORAL---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_tel_laboral_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['tel_laboral_fitsales'])) {
            array_push($relations['tel_laboral_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Teléfono Laboral Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----TEL MOVIL---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_tel_movil_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['tel_movil_fitsales'])) {
            array_push($relations['tel_movil_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Teléfono Móvil Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----DIRECCION---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_direccion_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['direccion_fitsales'])) {
            array_push($relations['direccion_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Dirección Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----LOCALIDAD---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_localidad_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['localidad_fitsales'])) {
            array_push($relations['localidad_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Localidad Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----CODIGO POSTAL---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_cod_postal_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['cod_postal_fitsales'])) {
            array_push($relations['cod_postal_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Código Postal Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----PROVINCIA---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_provincia_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['provincia_fitsales'])) {
            array_push($relations['provincia_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Provincia Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----DOCUMENTO---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_documento_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['documento_fitsales'])) {
            array_push($relations['documento_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Documento Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----FECHA NACIMIENTO---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_fec_nac_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['fec_nac_fitsales'])) {
            array_push($relations['fec_nac_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Fecha Nacimiento Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----SCORING---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_scoring_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['scoring_fitsales'])) {
            array_push($relations['scoring_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Scoring Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----TRACKING---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_tracking_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['tracking_fitsales'])) {
            array_push($relations['tracking_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Tracking Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----COMENTARIO---*/
        unset($args);
        $fieldname = 'wp_fitsales_matching_comentario_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'select',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'get_options_list' => $fieldlist,
            'value_type'=>'normal',
            'wp_data' => 'option',
            'post_id' => $id,
        );

        if (!in_array($fieldname, $relations['comentario_fitsales'])) {
            array_push($relations['comentario_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Comentario Fitsales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        /*----CAMPOS EXTRA---*/
        //ACA PUEDEN CONCATENARSE VARIOS CAMPOS NO FITSALES
        unset($args);
        $fieldname = 'wp_fitsales_matching_camposextra_' . $prefix . '_' . $id;

        $args = array (
            'type'      => 'textarea',
            'subtype'   => 'text',
            'id'    => $fieldname,
            'name'      => $fieldname,
            'required'  => false,
            'value_type'=>'normal',
            'wp_data' => 'option',
            //'pretip' => 'Agregue comentarios y/o campos adicionales del formulario.<br>Defina los datos de los campos del formulario entre llaves, ej. {sitio_web} y puede identificarlos con un texto, o separarlos con comas, etc.',
        );

        if (!in_array($fieldname, $relations['camposextra_fitsales'])) {
            array_push($relations['camposextra_fitsales'], $fieldname);
        }

        add_settings_field(
            $fieldname,
            'Datos Adicionales',
            array( $this, 'wp_fitsales_render_settings_field' ),
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            'wp_fitsales_section_matching_' . $prefix . '_' . $id,
            $args
        );

        register_setting(
            'wp_fitsales_settings_' . $prefix . '_' . $id,
            $fieldname
        );

        //texto final de datos adicionales
        add_settings_section(
            // ID used to identify this section and with which to register options
            'wp_fitsales_section_datos_adicionales' . $prefix . '_' . $id,
            // Title to be displayed on the administration page
            '',
            // Callback used to render the description of the section
            array( $this, 'wp_fitsales_section_datos_adicionales' ),
            // Page on which to add this section of options
            'wp_fitsales_settings_' . $prefix . '_' . $id
        );

        update_option('wp_fitsales_fields_relations', $relations);
    }


}
