<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://example.com
 * @since      1.0.0
 *
 * @package    WP_Fitsales
 * @subpackage WP_Fitsales/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    WP_Fitsales
 * @subpackage WP_Fitsales/public
 * @author     Your Name <email@example.com>
 */
class WP_Fitsales_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The protocol of the Fitsales endpoint.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $protocol
	 */
	private static $protocol;

	/**
	 * The remaining portion of the endpoint.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $endpointsuffix
	 */
	private static $endpointsuffix;

    /**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
        self::$protocol = 'https://';
        
        self::$endpointsuffix = WP_FITSALES_SANDBOX ? '.conectamospuntos.com/desaqa/registra_lead/json' : '.fitsales.com.ar/registra_lead/json';
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Plugin_Name_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Plugin_Name_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/wp-fitsales-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Plugin_Name_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Plugin_Name_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/wp-fitsales-public.js', array( 'jquery' ), $this->version, false );

	}

    public static function is_cf7_integrated() {
        return WP_Fitsales_Public::check_plugin_active('contact-form-7/wp-contact-form-7.php');
    }

    public static function is_wpforms_integrated() {
        return WP_Fitsales_Public::check_plugin_active('wpforms/wpforms.php');
    }

    public static function is_ninjaforms_integrated() {
        return WP_Fitsales_Public::check_plugin_active('ninja-forms/ninja-forms.php');
    }

    public static function getPrefixCF7() {
        return 'cf7';
    }

    public static function getPrefixWPForms() {
        return 'wpforms';
    }

    public static function getPrefixNinjaForms() {
        return 'ninjaforms';
    }

    public static function getBasicFormsWPForms() {
        $args = array('post_type' => 'wpforms', 'post_per_page' => -1);
        $Forms = get_posts( $args );

        foreach ($Forms as $Form) {
            $response[$Form->ID] = $Form->post_title;
        }

        return $response;
    }

    public static function getBasicFormsCF7() {
        $args = array('post_type' => 'wpcf7_contact_form', 'posts_per_page' => -1);
        $Forms = get_posts( $args );

        foreach ($Forms as $Form) {
            $response[$Form->ID] = $Form->post_title;
        }
        
        return $response;
    }

    public static function getBasicFormsNinjaForms() {
        if ( function_exists('ninja_forms_get_all_forms') ) {
            $nfs = ninja_forms_get_all_forms();
        }

        foreach ($nfs as $ninjaform) {
            $response[$ninjaform['id']] = $ninjaform['data']['form_title'];
        }

        return $response;
    }

    public static function getFormFieldListWPForms( $formid ) {
        $form = wpforms()->form->get( absint( $formid ) );
        $form_fields = json_decode($form->post_content);
        
        $fieldlist = [];
        foreach( $form_fields->fields as $key => $field) {
            $fieldlist[$key] = $field->label;
        }

        return $fieldlist;
    }

    public static function getFormFieldListCF7( $formid ) {
        if (!class_exists('WPCF7_ContactForm'))
            return false;
        
        $fieldlist = [];

        $ContactForm = WPCF7_ContactForm::get_instance( $formid );

        $form_fields = $ContactForm->scan_form_tags();

        foreach( $form_fields as $field) {
            if ($field->type != 'submit')
                $fieldlist[$field->name] = $field->name;
        }

        return $fieldlist;
    }

    public static function getFormFieldListNinjaForms( $formid ) {
        $form_fields = Ninja_Forms()->form( $formid )->get_fields();

        $fieldlist = [];
        foreach( $form_fields as $field) {
            if ($field->get_setting('type') != 'submit')
                $fieldlist[$field->get_setting('key')] = $field->get_setting('label');
        }

        return $fieldlist;
    }
    
    public function fitsales_connect_ninjaforms($form_data) {
        // @see https://developer.ninjaforms.com/codex/submission-processing-hooks/
        error_log('HOOK NINJA FORMS FUNCIONANDO');

        $activo = get_option('wp_fitsales_cfg_activo_' . $this->getPrefixNinjaForms() . '_' . $form_data['form_id']);
        
        if (!$activo) //si no está activo, no aborta, sólo deja de enviar lead a fitsales
            return;

        //DEBO VALIDAR: QUE HAYA CONECTOR
        if (!get_option('wp_fitsales_cfg_conector_' . $this->getPrefixNinjaForms() . '_' . $form_data['form_id']))
            return;
        
        $matched = $this->get_matched_fields($this->getPrefixNinjaForms(), $form_data['form_id']);

        //TODO: Verificar tambien que tenga nombre?
        if (!$matched)
            return;

        $fields2send = [];
        foreach($matched as $fieldname => $ninjaformsname) {
            $fields2send[$fieldname] = $form_data['fields_by_key'][$ninjaformsname]['value'] ?? '';
        }

        if (!$fields2send)
            return;
        
        $fieldname_campania = 'wp_fitsales_cfg_campania_' . $this->getPrefixNinjaForms() . '_' . $form_data['form_id'];
        $fieldname_medio = 'wp_fitsales_cfg_medio_' . $this->getPrefixNinjaForms()  . '_' . $form_data['form_id'];
        $fieldname_fuente = 'wp_fitsales_cfg_fuente_' . $this->getPrefixNinjaForms() . '_' . $form_data['form_id'];
        
        $var_campania = $this->get_field_variable($fieldname_campania);
        $var_medio = $this->get_field_variable($fieldname_medio);
        $var_fuente = $this->get_field_variable($fieldname_fuente);
        
        $fields2send['campania_fitsales'] =  $var_campania ?
                        $form_data['fields_by_key'][$var_campania[1]]['value'] : get_option($fieldname_campania);
        $fields2send['medio_fitsales'] =  $var_medio ?
                        $form_data['fields_by_key'][$var_medio[1]]['value'] : get_option($fieldname_medio);
        $fields2send['fuente_fitsales'] =  $var_fuente ?
                        $form_data['fields_by_key'][$var_fuente[1]]['value'] : get_option($fieldname_fuente);
                
        $fieldname_camposextra = 'wp_fitsales_matching_camposextra_' . $this->getPrefixNinjaForms() . '_' . $form_data['form_id'];
        
        $vars_camposextra = $this->get_field_variables($fieldname_camposextra);
        if ($vars_camposextra)
        {
            unset($fields2send['camposextra_fitsales']); //no es campo fitsales

            //REEMPLAZO VALORES
            $camposextra = get_option($fieldname_camposextra);
            foreach($vars_camposextra[0] as $key => $replace) {
                $camposextra = str_replace($replace, $form_data['fields_by_key'][$vars_camposextra[1][$key]]['value'], $camposextra);
            }

            if ( $fields2send['comentario_fitsales'] )
                $fields2send['comentario_fitsales'] = strip_tags($fields2send['comentario_fitsales'],'<br />') . '<br />---<br />' . $camposextra;    
            else
                $fields2send['comentario_fitsales'] = $camposextra;
        }
                
        $conector = get_option('wp_fitsales_cfg_conector_' . $this->getPrefixNinjaForms() . '_' . $form_data['form_id']);

        $this->send_lead_to_crm($conector, $fields2send); //$extrafields2send
    }

    public function fitsales_connect_wpforms($fields, $entry, $form_data, $entry_id) {
        //La accion wp_forms_process_complete es POSTERIOR al envío de correo y guardado en DB
        //La acción wp_forms_process es ANTERIOR al envío de correo y guardado en DB, pero tambien es anterior a todas las validaciones de campo....
        // @see https://wpforms.com/developers/wpforms_process_complete/
        error_log('HOOK WPFORMS FUNCIONANDO');
        error_log('Fields: ' . print_r($fields, true));
        error_log('Entry: ' . print_r($entry, true));

        $activo = get_option('wp_fitsales_cfg_activo_' . $this->getPrefixWPForms() . '_' . absint( $form_data[ 'id' ] ));

        if (!$activo) //idem ninja forms, no abortan el correo, sólo dejan de enviar lead a fitsales.
            return;

        //DEBO VALIDAR: QUE HAYA CONECTOR
        if (!get_option('wp_fitsales_cfg_conector_' . $this->getPrefixWPForms() . '_' . absint( $form_data[ 'id' ] )))
            return;

        $matched = $this->get_matched_fields($this->getPrefixWPForms(), absint( $form_data[ 'id' ] ));

        //TODO: Verificar tambien que tenga nombre?
        if (!$matched)
            return;

        error_log('Matched: ' . print_r($matched, true));

        $fields2send = [];
        foreach($matched as $fieldname => $wpformsid) {
            $fields2send[$fieldname] = $fields[$wpformsid]['value'] ?? '';
        }

        if (!$fields2send)
            return;
        
        $fieldname_campania = 'wp_fitsales_cfg_campania_' . $this->getPrefixWPForms() . '_' . absint( $form_data[ 'id' ] );
        $fieldname_medio = 'wp_fitsales_cfg_medio_' . $this->getPrefixWPForms()  . '_' . absint( $form_data[ 'id' ] );
        $fieldname_fuente = 'wp_fitsales_cfg_fuente_' . $this->getPrefixWPForms() . '_' . absint( $form_data[ 'id' ] );
        
        $var_campania = $this->get_field_variable($fieldname_campania);
        $var_medio = $this->get_field_variable($fieldname_medio);
        $var_fuente = $this->get_field_variable($fieldname_fuente);
        
        $fields2send['campania_fitsales'] =  $var_campania ?
                                                $fields[$var_campania[1]]['value'] : get_option($fieldname_campania);
        $fields2send['medio_fitsales'] =  $var_medio ?
                                                $fields[$var_medio[1]]['value'] : get_option($fieldname_medio);
        $fields2send['fuente_fitsales'] =  $var_fuente ?
                                                $fields[$var_fuente[1]]['value'] : get_option($fieldname_fuente);

        $fieldname_camposextra = 'wp_fitsales_matching_camposextra_' . $this->getPrefixWPForms() . '_' . absint( $form_data[ 'id' ] );
        $vars_camposextra = $this->get_field_variables($fieldname_camposextra);
        if ($vars_camposextra)
        {
            unset($fields2send['camposextra_fitsales']); //no es campo fitsales

            //REEMPLAZO VALORES
            $camposextra = get_option($fieldname_camposextra);
            foreach($vars_camposextra[0] as $key => $replace) {
                $camposextra = str_replace($replace, $fields[$vars_camposextra[1][$key]]['value'], $camposextra);
            }

            if ( $fields2send['comentario_fitsales'] )
                $fields2send['comentario_fitsales'] = strip_tags($fields2send['comentario_fitsales'],'<br />') . '<br />---<br />' . $camposextra;    
            else
                $fields2send['comentario_fitsales'] = $camposextra;
        }

        $conector = get_option('wp_fitsales_cfg_conector_' . $this->getPrefixWPForms() . '_' . absint( $form_data[ 'id' ] ));
        
        $this->send_lead_to_crm($conector, $fields2send); //$extrafields2send
    }

    public function fitsales_connect_cf7($wpcf7, &$abort, $object) {
        
        $activo = get_option('wp_fitsales_cfg_activo_' . $this->getPrefixCF7() . '_' . $wpcf7->id());

        if (!$activo)
            return;

        //DEBO VALIDAR: QUE HAYA CONECTOR
        if (!get_option('wp_fitsales_cfg_conector_' . $this->getPrefixCF7() . '_' . $wpcf7->id()))
            return;
                        
        $submission = WPCF7_Submission::get_instance();

        // En caso de que no haya valores salgo de la función
        if( empty( $submission ) )
            return;

        $matched = $this->get_matched_fields($this->getPrefixCF7(), $wpcf7->id());

        //TODO: Verificar tambien que tenga nombre?
        if (!$matched)
            return;

        $formulario = array();
        $formulario['posted_data'] = $submission->get_posted_data();

        date_default_timezone_set('America/Argentina/Buenos_Aires');
        
        $fields2send = [];
        foreach($matched as $fieldname => $cf7name) {
            $fields2send[$fieldname] = $formulario['posted_data'][$cf7name] ?? '';
        }

        $fieldname_utm = 'wp_fitsales_cfg_utm_' . $this->getPrefixCF7() . '_' . $wpcf7->id();
        
        $fieldname_campania = 'wp_fitsales_cfg_campania_' . $this->getPrefixCF7() . '_' . $wpcf7->id();
        $fieldname_medio = 'wp_fitsales_cfg_medio_' . $this->getPrefixCF7()  . '_' . $wpcf7->id();
        $fieldname_fuente = 'wp_fitsales_cfg_fuente_' . $this->getPrefixCF7() . '_' . $wpcf7->id();
        
        //$var_campania = $this->get_field_variable($fieldname_campania);
        //$var_medio = $this->get_field_variable($fieldname_medio);
        //$var_fuente = $this->get_field_variable($fieldname_fuente);
        $var_campania = 'utm_campaign';
        $var_medio = 'utm_medium';
        $var_fuente = 'utm_source';
        
        if (!$fieldname) {
            $fields2send['campania_fitsales'] = get_option($fieldname_campania);
            $fields2send['medio_fitsales'] = get_option($fieldname_medio);
            $fields2send['fuente_fitsales'] = get_option($fieldname_fuente);
        } else {

            $fields2send['campania_fitsales'] = !empty($formulario['posted_data'][$var_campania]) ?
                                                    $formulario['posted_data'][$var_campania]
                                                    :
                                                    get_option($fieldname_campania);

            $fields2send['medio_fitsales'] = !empty($formulario['posted_data'][$var_medio]) ?
                                                $formulario['posted_data'][$var_medio]
                                                :
                                                get_option($fieldname_medio);

            $fields2send['fuente_fitsales'] =  !empty($formulario['posted_data'][$var_fuente]) ?
                                                    $formulario['posted_data'][$var_fuente]
                                                    :
                                                    get_option($fieldname_fuente);
        }

        $fieldname_camposextra = 'wp_fitsales_matching_camposextra_' . $this->getPrefixCF7() . '_' . $wpcf7->id();
        $vars_camposextra = $this->get_field_variables($fieldname_camposextra);
        if ($vars_camposextra)
        {
            unset($fields2send['camposextra_fitsales']); //no es campo fitsales

            //REEMPLAZO VALORES
            $camposextra = get_option($fieldname_camposextra);
            error_log('Campos Extra option: ' . print_r($camposextra,true));
            error_log('Variables Campos Extra: ' . print_r($vars_camposextra, true));
            error_log('Valores Form: ' . print_r( $formulario['posted_data'], true ) );
            foreach($vars_camposextra[0] as $key => $replace) {
                if (is_array($formulario['posted_data'][$vars_camposextra[1][$key]]))
                    $value = $formulario['posted_data'][$vars_camposextra[1][$key]][0];
                else
                    $value = $formulario['posted_data'][$vars_camposextra[1][$key]];

                $camposextra = str_replace($replace, $value, $camposextra);
            }

            if ( !empty($fields2send['comentario_fitsales']) )
                $fields2send['comentario_fitsales'] = strip_tags($fields2send['comentario_fitsales'],'<br />') . '<br />---<br />' . $camposextra;    
            else
                $fields2send['comentario_fitsales'] = $camposextra;
        }

        $conector = get_option('wp_fitsales_cfg_conector_' . $this->getPrefixCF7() . '_' . $wpcf7->id());

        $this->send_lead_to_crm($conector, $fields2send);
    }

    private function get_field_variables($fieldname)
    {
        $content = get_option($fieldname);
        preg_match_all('/\{(.*?)\}/', $content, $matches);
        
        return $matches;
    }
    
    private function get_field_variable($fieldname)
    {
        $content = get_option($fieldname);
        preg_match('/\{(.*?)\}/', $content, $matches);
        
        return $matches;
    }

    public static function get_matched_fields($prefix, $formid) {
        $relations = get_option('wp_fitsales_fields_relations');
        $matched = [];

        foreach ($relations as $fitsales_fieldname => $wp_fieldnames) {
            $subname = str_replace('_fitsales', '', $fitsales_fieldname);
            if (get_option('wp_fitsales_matching_' . $subname. '_' . $prefix . '_' . $formid ))
                $matched[$fitsales_fieldname] = get_option('wp_fitsales_matching_' . $subname. '_' . $prefix . '_' . $formid );
        }

        return $matched;
    }

    public static function getEndpoint() {
        return self::$protocol . get_option('wp_fitsales_endpoint') . self::$endpointsuffix;
    }

    private function send_lead_to_crm(int $conector, array $lead, array $comments = [])
    {
        $url = $this->getEndpoint();
        $lead += [
            'token' => get_option('wp_fitsales_token'),
            'conector_fitsales' => $conector,
        ];

        error_log('VALORES plugin: ' . print_r( $lead, true));
        error_log('URL: ' . $url);
        error_log('Token: ' . get_option('wp_fitsales_token'));
        error_log('Conector: ' . $conector);
        
        if ($comments) {
            if (empty($lead['comentario_fitsales'])) {
                $lead['comentario_fitsales'] = '';
            } else {
                $lead['comentario_fitsales'] .= "\n";
            }
    
            foreach ($comments as $caption => $value) {
                $lead['comentario_fitsales'] .= sprintf("%s %s\n", $caption, $value);
            }
        }
    
        $result = @file_get_contents(
            $url,
            false,
            stream_context_create([
                'http' => [
                    'method' => 'POST',
                    'header' => 'Content-Type: application/x-www-form-urlencoded',
                    'content' => http_build_query($lead),
                ],
                'ssl' => [
                    'verify_peer' => false,
                    'verify_peer_name' => false,
                ],
            ])
        );
    
        return $result == 0;
    }
    
    /**
     * Check if a plugin is active
     *
     * @param mixed $plugin_path. Ex: plugin_name/plugin_name.php
     * 
     * @return boolean
     * 
     */
    public static function check_plugin_active($plugin_path)
    {
        return in_array( $plugin_path, apply_filters('active_plugins', get_option('active_plugins')));
    }

}
