<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              http://fitsales.net
 * @since             1.0.0
 * @package           WP_Fitsales
 *
 * @wordpress-plugin
 * Plugin Name:       Fitsales Lead Capture
 * Plugin URI:        https://fitsales.net/integraciones
 * Description:       Recibe leads automáticamente en FitCRM desde los formularios de tu sitio web. Admite los gestores de formularios más utilizados (Contact Form 7, WP Forms y Ninja Forms).
 * Version:           1.0
 * Author:            Fitsales
 * Author URI:        https://fitsales.net/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       wp-fitsales
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'WP_FITSALES_VERSION', '1.0' );

/**
 * Sandbox plugin mode?.
 */
define( 'WP_FITSALES_SANDBOX', false );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-plugin-name-activator.php
 */
function activate_wp_fitsales() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-wp-fitsales-activator.php';
	WP_Fitsales_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-plugin-name-deactivator.php
 */
function deactivate_wp_fitsales() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-wp-fitsales-deactivator.php';
	WP_Fitsales_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_wp_fitsales' );
register_deactivation_hook( __FILE__, 'deactivate_wp_fitsales' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-wp-fitsales.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_plugin_name() {

	$plugin = new WP_Fitsales();
	$plugin->run();

}
run_plugin_name();
